
CONSTRUCT_BLOCK_TOOLBAR  = "wagtail_fedit.construct_block_toolbar"


CONSTRUCT_FIELD_TOOLBAR  = "wagtail_fedit.construct_field_toolbar"

"""
Register a custom renderer for a type.
"""
REGISTER_TYPE_RENDERER   = "wagtail_fedit.register_type_renderer"

"""
Register a custom renderer for a field.
This can be used to say; automatically render your RichText value. (wagtail_hooks/renderers.py)
"""
REGISTER_FIELD_RENDERER  = "wagtail_fedit.register_field_renderer"


EXCLUDE_FROM_RELATED_FORMS = "wagtail_fedit.exclude_related_forms"